@@ -18,7 +18,6 @@ from typing import Any, Dict, List, Mapping, Optional, Set
 import numpy as np
 from torch.utils.data import Dataset
 
-from fastestimator.dataset import BatchDataset
 from fastestimator.op.numpyop.numpyop import NumpyOp, forward_numpyop
 from fastestimator.util.traceability_util import traceable
 from fastestimator.util.util import pad_batch
@@ -35,12 +34,15 @@ class _DelayedDeepDict(dict):
     Args:
         base: The dictionary to be wrapped.
     """
+    warned: Set[str] = set()
+
     def __init__(self, base: Dict[str, Any]):
         super().__init__()
         self.base = base
+        self.mask = set()
 
     def __getitem__(self, key: str) -> Any:
-        if key not in self and key in self.base:
+        if key not in self and key in self.base and key not in self.mask:
             item = self.base[key]
             if isinstance(item, np.ndarray):
                 # We'll avoid copying ndarrays for speed/memory savings. The forward_numpyop function will copy later if
@@ -57,6 +59,7 @@ class _DelayedDeepDict(dict):
         if key in self or key not in self.base:
             # 'key not in base' to raise errors when key doesn't exist
             super().__delitem__(key)
+        self.mask.add(key)  # Mask the key so that it cannot be drawn out of the base
 
     def finalize(self, retain: Optional[Set[str]] = None, deep_remainder: bool = True) -> None:
         """Finish migrating the data from the original dictionary into this one.
@@ -66,7 +69,13 @@ class _DelayedDeepDict(dict):
             deep_remainder: Whether to deep copy any keys which have not yet been copied.
         """
         for key in self.base:
+            if key in self.mask:
+                continue
             if retain and key not in retain:
+                if not key in self.warned:
+                    self.warned.add(key)
+                    print("FastEstimator-Warn: the key '{}' is being pruned since it is unused outside of the Pipeline."
+                          " To prevent this, you can declare the key as an input of a Trace or TensorOp.".format(key))
                 continue
             if key not in self:
                 if deep_remainder:
@@ -75,6 +84,10 @@ class _DelayedDeepDict(dict):
                     self[key] = self.base[key]
         if retain:
             for key in self.keys() - retain:
+                if not key in self.warned:
+                    self.warned.add(key)
+                    print("FastEstimator-Warn: the key '{}' is being pruned since it is unused outside of the Pipeline."
+                          " To prevent this, you can declare the key as an input of a Trace or TensorOp.".format(key))
                 del self[key]
         self.base = {}
         # We need to mark all of the arrays as writeable again to avoid warnings from pytorch. Once torch wraps the
@@ -98,15 +111,17 @@ class OpDataset(Dataset):
         output_keys: What keys can be produced from pipeline. If None, all keys will be considered.
         deep_remainder: Whether data which is not modified by Ops should be deep copied or not. This argument is used to
             help with RAM management, but end users can almost certainly ignore it.
+        shuffle: Whether to shuffle batched datasets every epoch.
     """
     def __init__(self,
                  dataset: Dataset,
                  ops: List[NumpyOp],
                  mode: str,
                  output_keys: Optional[Set[str]] = None,
-                 deep_remainder: bool = True) -> None:
+                 deep_remainder: bool = True,
+                 shuffle: bool = True) -> None:
         self.dataset = dataset
-        if isinstance(self.dataset, BatchDataset):
+        if hasattr(self.dataset, "reset_index_maps") and shuffle:
             self.dataset.reset_index_maps()
         self.ops = ops
         self.mode = mode
@@ -122,11 +137,12 @@ class OpDataset(Dataset):
         Returns:
             The data dictionary from the specified index, with transformations applied.
         """
-        if isinstance(self.dataset, BatchDataset):
+        item = self.dataset[index]
+        if isinstance(item, list):
             # BatchDataset may randomly sample the same elements multiple times, so need to avoid reprocessing
             unique_samples = {}  # id: idx
             results = []
-            for idx, data in enumerate(self.dataset[index]):
+            for idx, data in enumerate(item):
                 data_id = id(data)
                 if data_id not in unique_samples:
                     data = _DelayedDeepDict(data)
@@ -136,11 +152,11 @@ class OpDataset(Dataset):
                     unique_samples[data_id] = idx
                 else:
                     results.append(results[unique_samples[data_id]])
-            if self.dataset.pad_value is not None:
+            if hasattr(self.dataset, "pad_value") and self.dataset.pad_value is not None:
                 pad_batch(results, self.dataset.pad_value)
-            results = {key: np.array([item[key] for item in results]) for key in results[0]}
+            results = {key: np.array([result[key] for result in results]) for key in results[0]}
         else:
-            results = _DelayedDeepDict(self.dataset[index])
+            results = _DelayedDeepDict(item)
             forward_numpyop(self.ops, results, {'mode': self.mode})
             results.finalize(retain=self.output_keys, deep_remainder=self.deep_remainder)
         return results
