@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
+import gc
 import os
 import tempfile
 from collections import ChainMap
@@ -108,7 +109,13 @@ class BaseNetwork:
             all_items = self.ops + self.postprocessing
         return all_items
 
-    def load_epoch(self, mode: str, epoch: int, output_keys: Optional[Set[str]] = None, warmup: bool = False) -> None:
+    def load_epoch(self,
+                   mode: str,
+                   epoch: int,
+                   ds_id: Union[str, None],
+                   output_keys: Optional[Set[str]] = None,
+                   warmup: bool = False,
+                   eager: bool = False) -> None:
         """Prepare the network to run a given epoch and mode.
 
         This method is necessary since schedulers and op mode restrictions may result in different computation graphs
@@ -117,22 +124,30 @@ class BaseNetwork:
         Args:
             mode: The mode to prepare to execute. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch to prepare to execute.
+            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
             output_keys: What keys can be moved from the GPU back to the CPU after executing a step.
             warmup: Whether to prepare to execute it warmup mode or not (end users can likely ignore this argument).
+            eager: Whether to run the training in eager mode. This is only related to TensorFlow training because
+                PyTorch by nature is always in eager mode.
         """
-        self.effective_inputs[mode] = self.get_effective_input_keys(mode, epoch)
-        self.effective_outputs[mode] = self.get_all_output_keys(mode, epoch)
+        self.effective_inputs[mode] = self.get_effective_input_keys(mode, epoch, ds_id)
+        self.effective_outputs[mode] = self.get_all_output_keys(mode, epoch, ds_id)
         if output_keys:
             self.effective_outputs[mode] = self.effective_outputs[mode].intersection(
-                output_keys) | self._get_effective_postprocessing_input_keys(mode, epoch)
-        self.epoch_ops = get_current_items(self.ops, mode, epoch)
-        self.epoch_postprocessing = get_current_items(self.postprocessing, mode, epoch)
+                output_keys) | self._get_effective_postprocessing_input_keys(mode, epoch, ds_id)
+        self.epoch_ops = get_current_items(self.ops, mode, epoch, ds_id=ds_id)
+        self.epoch_postprocessing = get_current_items(self.postprocessing, mode, epoch, ds_id=ds_id)
         self.epoch_models = set.union(*[op.get_fe_models() for op in self.epoch_ops])
         gradient_ops = [op for op in self.epoch_ops if op.fe_retain_graph() is not None]
         for idx, gradient_op in enumerate(gradient_ops):
             gradient_op.fe_retain_graph(idx != len(gradient_ops) - 1)
         self.epoch_state = {
-            "warmup": warmup, "mode": mode, "req_grad": len(gradient_ops) > 0, "epoch": epoch, "deferred": {}
+            "warmup": warmup,
+            "mode": mode,
+            "req_grad": len(gradient_ops) > 0,
+            "epoch": epoch,
+            "deferred": {},
+            "eager": eager
         }
         # warmup: bool, mode: str, req_grad: bool, epoch: int, deferred: Dict[str, List[Callable]]]
         for model in self.epoch_models:
@@ -158,52 +173,55 @@ class BaseNetwork:
             loss_keys |= op.get_fe_loss_keys()
         return loss_keys
 
-    def get_effective_input_keys(self, mode: str, epoch: int) -> Set[str]:
+    def get_effective_input_keys(self, mode: str, epoch: int, ds_id: Optional[str] = None) -> Set[str]:
         """Determine which keys need to be provided as input to the network during the given `epoch`.
 
         Args:
             mode: The execution mode to consider. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch number to consider for determining inputs.
+            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
 
         Returns:
             The necessary inputs for the network to execute the given `epoch` and `mode`.
         """
         input_keys = set()
         produced_keys = set()
-        for op in get_current_items(self.ops + self.postprocessing, mode, epoch):
+        for op in get_current_items(self.ops + self.postprocessing, mode, epoch, ds_id=ds_id):
             input_keys.update(set(key for key in op.inputs if key not in produced_keys))
             produced_keys.update(op.outputs)
         return input_keys
 
-    def _get_effective_postprocessing_input_keys(self, mode: str, epoch: int) -> Set[str]:
+    def _get_effective_postprocessing_input_keys(self, mode: str, epoch: int, ds_id: Optional[str] = None) -> Set[str]:
         """Determine which keys need to be provided as input to the postprocessing during the given `epoch`.
 
         Args:
             mode: The execution mode to consider. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch number to consider for determining inputs.
+            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
 
         Returns:
             The necessary inputs for the postprocessing to execute the given `epoch` and `mode`.
         """
         input_keys = set()
         produced_keys = set()
-        for op in get_current_items(self.postprocessing, mode, epoch):
+        for op in get_current_items(self.postprocessing, mode, epoch, ds_id=ds_id):
             input_keys.update(set(key for key in op.inputs if key not in produced_keys))
             produced_keys.update(op.outputs)
         return input_keys
 
-    def get_all_output_keys(self, mode: str, epoch: int) -> Set[str]:
+    def get_all_output_keys(self, mode: str, epoch: int, ds_id: Optional[str] = None) -> Set[str]:
         """Get all of the keys that will be generated by the network during the given `epoch` and `mode`.
 
         Args:
             mode: The execution mode to consider. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch number to consider when searching for outputs.
+            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
 
         Returns:
             The keys that will be generated by the network's Ops during the `epoch` for the given `mode`.
         """
         output_keys = set()
-        for op in get_current_items(self.ops + self.postprocessing, mode, epoch):
+        for op in get_current_items(self.ops + self.postprocessing, mode, epoch, ds_id=ds_id):
             output_keys.update(op.outputs)
         return output_keys
 
@@ -259,18 +277,20 @@ class BaseNetwork:
         """
         raise NotImplementedError
 
-    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1) -> Dict[str, Any]:
+    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1, ds_id: Optional[
+                                                                                      str] = None) -> Dict[str, Any]:
         """Run a forward step through the Network on an element of data.
 
         Args:
             data: The element to data to use as input.
             mode: The mode in which to run the transform. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch in which to run the transform.
+            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
 
         Returns:
             prediction_data overlaid on the input `data`.
         """
-        self.load_epoch(mode, epoch, warmup=False)
+        self.load_epoch(mode, epoch, ds_id, warmup=False, eager=True)
         data = to_tensor(data, target_type=self.target_type)
         data, prediction = self.run_step(data)
         self.unload_epoch()
@@ -314,6 +334,7 @@ def Network(
         AssertionError: If TensorFlow and PyTorch models are mixed, or if no models are provided.
         ValueError: If a model is provided whose type cannot be identified as either TensorFlow or PyTorch.
     """
+    ops = to_list(ops)
     models = _collect_models(ops)
     framework = set()
     model_names = set()
@@ -361,7 +382,13 @@ class TorchNetwork(BaseNetwork):
                          ops=ops,
                          postprocessing=postprocessing)
 
-    def load_epoch(self, mode: str, epoch: int, output_keys: Optional[Set[str]] = None, warmup: bool = False) -> None:
+    def load_epoch(self,
+                   mode: str,
+                   epoch: int,
+                   ds_id: Union[str, None],
+                   output_keys: Optional[Set[str]] = None,
+                   warmup: bool = False,
+                   eager: bool = False) -> None:
         """Prepare the network to run a given epoch and mode.
 
         This method is necessary since schedulers and op mode restrictions may result in different computation graphs
@@ -370,10 +397,13 @@ class TorchNetwork(BaseNetwork):
         Args:
             mode: The mode to prepare to execute. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch to prepare to execute.
+            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
             output_keys: What keys must be moved from the GPU back to the CPU after executing a step.
             warmup: Whether to prepare to execute it warmup mode or not (end users can likely ignore this argument).
+            eager: Whether to run the training in eager mode. This is only related to TensorFlow training because
+                PyTorch by nature is always in eager mode.
         """
-        super().load_epoch(mode, epoch, output_keys, warmup)
+        super().load_epoch(mode=mode, epoch=epoch, ds_id=ds_id, output_keys=output_keys, warmup=warmup, eager=eager)
         if self.device.type == "cuda":
             for model in self.epoch_models:
                 # move model variables to gpu
@@ -541,7 +571,13 @@ class TFNetwork(BaseNetwork):
     ) -> None:
         super().__init__(target_type='tf', device=None, ops=ops, postprocessing=postprocessing)
 
-    def load_epoch(self, mode: str, epoch: int, output_keys: Optional[Set[str]] = None, warmup: bool = False) -> None:
+    def load_epoch(self,
+                   mode: str,
+                   epoch: int,
+                   ds_id: Union[None, str],
+                   output_keys: Optional[Set[str]] = None,
+                   warmup: bool = False,
+                   eager: bool = False) -> None:
         """Prepare the network to run a given epoch and mode.
 
         This method is necessary since schedulers and op mode restrictions may result in different computation graphs
@@ -550,10 +586,13 @@ class TFNetwork(BaseNetwork):
         Args:
             mode: The mode to prepare to execute. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch to prepare to execute.
+            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
             output_keys: What keys must be moved from the GPU back to the CPU after executing a step.
             warmup: Whether to prepare to execute it warmup mode or not (end users can likely ignore this argument).
+            eager: Whether to run the training in eager mode. This is only related to TensorFlow training because
+                PyTorch by nature is always in eager mode.
         """
-        super().load_epoch(mode, epoch, output_keys, warmup)
+        super().load_epoch(mode=mode, epoch=epoch, ds_id=ds_id, output_keys=output_keys, warmup=warmup, eager=eager)
         # Don't cause a re-trace just because epoch changed
         self.epoch_state["epoch"] = tf.convert_to_tensor(self.epoch_state["epoch"])
         # Need to re-trace the TF graph if the optimizer is changing due to scheduling:
@@ -577,7 +616,7 @@ class TFNetwork(BaseNetwork):
         batch_in = self._get_effective_batch_input(batch, mode)
         strategy = tf.distribute.get_strategy()
         if isinstance(strategy, tf.distribute.MirroredStrategy):
-            if self.epoch_state["warmup"] == "debug":
+            if self.epoch_state["eager"]:
                 prediction = strategy.run(
                     self._forward_step_eager,
                     args=(batch_in, self.epoch_state, self.epoch_ops, to_list(self.effective_outputs[mode])))
@@ -588,7 +627,7 @@ class TFNetwork(BaseNetwork):
             batch = self._per_replica_to_global(batch)
             prediction = self._per_replica_to_global(prediction)
         else:
-            if self.epoch_state["warmup"] == "debug":
+            if self.epoch_state["eager"]:
                 prediction = self._forward_step_eager(batch_in,
                                                       self.epoch_state,
                                                       self.epoch_ops,
@@ -676,7 +715,7 @@ class TFNetwork(BaseNetwork):
                 prediction[key] = batch[key]
         return prediction
 
-    @tf.function
+    @tf.function(experimental_relax_shapes=True)
     def _forward_step_static(self,
                              batch: Dict[str, Any],
                              state: Dict[str, Any],
@@ -706,13 +745,14 @@ class TFNetwork(BaseNetwork):
                 prediction[key] = batch[key]
         return prediction
 
-    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1) -> Dict[str, Any]:
+    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1, ds_id: Optional[str] = None) -> Dict[str, Any]:
         """Run a forward step through the Network on an element of data.
 
         Args:
             data: The element to data to use as input.
             mode: The mode in which to run the transform. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch in which to run the transform.
+            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
 
         Returns:
             (batch_data, prediction_data)
@@ -726,7 +766,7 @@ class TFNetwork(BaseNetwork):
                 data = self._fill_batch(data, num_devices - batch_size)
                 sub_sample = True
             data = next(iter(strategy.experimental_distribute_dataset(tf.data.Dataset.from_tensors(data))))
-        results = super().transform(data, mode, epoch)
+        results = super().transform(data, mode, epoch, ds_id=ds_id)
         if sub_sample:
             results = self._subsample_data(results, batch_size)
         return results
@@ -822,33 +862,26 @@ def build(model_fn: Callable[[], Union[Model, List[Model]]],
 
     if not hasattr(build, "count"):
         build.count = 0
-    # Tensorflow models require setting global policies prior to model creation. Since there is no way to know the
-    # framework of the model ahead of time, set the policy for both tf and pytorch here.
+    # The following garbage collection is needed for if a TF model was running, but then died due to an exception being
+    # thrown, but the exception was then caught, whereupon the user wanted to switch to a pytorch model instead. Absent
+    # this collection, you would see: "Failed setting context: CUDA_ERROR_NOT_INITIALIZED: initialization error". This
+    # would be followed by the death of the pytorch multi-processor which would report something like the following:
+    # RuntimeError: DataLoader worker (pid 4225) is killed by signal: Aborted.
+    # RuntimeError: DataLoader worker (pid(s) 4225, 4226, 4227) exited unexpectedly
+    gc.collect()
+    # tensorflow models requires setting global policies prior to model creation. Since there is no way to know the
+    # framework of model, setting the policy for both tf and pytorch here.
     if mixed_precision:
         mixed_precision_tf.set_global_policy(mixed_precision_tf.Policy('mixed_float16'))
     else:
         mixed_precision_tf.set_global_policy(mixed_precision_tf.Policy('float32'))
-    models, optimizer_fn = to_list(model_fn()), to_list(optimizer_fn)
-    # fill optimizer
-    if not optimizer_fn:
-        optimizer_fn = [None]
-    # check framework
-    if isinstance(models[0], tf.keras.Model):
-        framework = "tf"
-    elif isinstance(models[0], torch.nn.Module):
-        framework = "torch"
-    else:
-        raise ValueError("unrecognized model format: {}".format(type(models[0])))
-    # multi-gpu handling
     if torch.cuda.device_count() > 1:
-        if framework == "tf" and not isinstance(tf.distribute.get_strategy(), tf.distribute.MirroredStrategy):
+        if not isinstance(tf.distribute.get_strategy(), tf.distribute.MirroredStrategy):
             tf.distribute.experimental_set_strategy(tf.distribute.MirroredStrategy())
-            models = to_list(model_fn())
-        if framework == "torch":
-            models = [torch.nn.DataParallel(model) for model in models]
-    # mark models with its mixed_precision flag
-    for model in models:
-        model.mixed_precision = mixed_precision
+    models, optimizer_fn = to_list(model_fn()), to_list(optimizer_fn)
+    # fill optimizers if optimizer_fn is None
+    if not optimizer_fn:
+        optimizer_fn = [None] * len(models)
     # generate names
     if not model_name:
         model_name = _generate_model_names(len(models))
@@ -862,7 +895,7 @@ def build(model_fn: Callable[[], Union[Model, List[Model]]],
         "Found inconsistency in number of models, optimizers, model_name or weights"
     # create optimizer
     for idx, (model, optimizer_def, weight, name) in enumerate(zip(models, optimizer_fn, weights_path, model_name)):
-        models[idx] = trace_model(_fe_compile(model, optimizer_def, weight, name, framework, mixed_precision),
+        models[idx] = trace_model(_fe_compile(model, optimizer_def, weight, name, mixed_precision),
                                   model_idx=idx if len(models) > 1 else -1,
                                   model_fn=model_fn,
                                   optimizer_fn=optimizer_def,
@@ -876,7 +909,6 @@ def _fe_compile(model: Model,
                 optimizer_fn: Union[str, Scheduler, Callable, None],
                 weight: Union[str, None],
                 name: str,
-                framework: str,
                 mixed_precision: bool) -> Model:
     """A function to bundle models with their optimizers.
 
@@ -885,13 +917,23 @@ def _fe_compile(model: Model,
         optimizer_fn: The optimizer to be associated with the given `model`.
         weight: A path to weights to be associated with the `model`.
         name: The name of the model.
-        framework: Which backend framework should be associated with this model (either 'tf' or 'torch').
         mixed_precision: Whether to enable mixed-precision training.
 
     Returns:
         The `model` combined with its optimizer, weights, and name. Models will also have an 'fe_compiled' flag to
         indicate that they were built via this function.
     """
+    if isinstance(model, tf.keras.Model):
+        framework = "tf"
+    elif isinstance(model, torch.nn.Module):
+        framework = "torch"
+    else:
+        raise ValueError("unrecognized model format: {}".format(type(model)))
+    # torch multi-gpu handling
+    if framework == "torch" and torch.cuda.device_count() > 1:
+        model = torch.nn.DataParallel(model)
+    # mark models with its mixed_precision flag
+    model.mixed_precision = mixed_precision
     if isinstance(optimizer_fn, EpochScheduler):
         for epoch, optimizer_def in optimizer_fn.epoch_dict.items():
             optimizer_fn.epoch_dict[epoch] = _build_optimizer(optimizer_def, model, framework, mixed_precision)
@@ -907,16 +949,13 @@ def _fe_compile(model: Model,
         model.input_spec = None  # this is to handle a behavior change in tf 2.4.1 that enforces a input shape check
     model.optimizer = optimizer_fn
     model.fe_compiled = True
-
     if weight:
         if weight.startswith(GOOGLE_DRIVE_URL):
             tmp_dir = tempfile.mkdtemp()
             file_name = gdown.download(weight, quiet=False)
             os.rename(os.path.join('./', file_name), os.path.join(tmp_dir, file_name))
             weight = gdown.download(weight, os.path.join(tmp_dir, file_name), quiet=False)
-
         load_model(model, weight)
-
     model.model_name = name
     return model
 
