@@ -34,10 +34,15 @@ class Average(TensorOp):
         mode: What mode(s) to execute this Op in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
+        ds_id: What dataset id(s) to execute this Op in. To execute regardless of ds_id, pass None. To execute in all
+            ds_ids except for a particular one, you can pass an argument like "!ds1".
     """
-    def __init__(self, inputs: Union[str, Iterable[str]], outputs: str, mode: Union[None, str,
-                                                                                    Iterable[str]] = None) -> None:
-        super().__init__(inputs=inputs, outputs=outputs, mode=mode)
+    def __init__(self,
+                 inputs: Union[str, Iterable[str]],
+                 outputs: str,
+                 mode: Union[None, str, Iterable[str]] = None,
+                 ds_id: Union[None, str, Iterable[str]] = None) -> None:
+        super().__init__(inputs=inputs, outputs=outputs, mode=mode, ds_id=ds_id)
         self.in_list, self.out_list = True, False
 
     def forward(self, data: List[Tensor], state: Dict[str, Any]) -> Tensor:
