@@ -33,9 +33,14 @@ class Watch(TensorOp):
         mode: What mode(s) to execute this Op in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
+        ds_id: What dataset id(s) to execute this Op in. To execute regardless of ds_id, pass None. To execute in all
+            ds_ids except for a particular one, you can pass an argument like "!ds1".
     """
-    def __init__(self, inputs: Union[None, str, Iterable[str]], mode: Union[None, str, Iterable[str]] = None) -> None:
-        super().__init__(inputs=inputs, outputs=inputs, mode=mode)
+    def __init__(self,
+                 inputs: Union[None, str, Iterable[str]],
+                 mode: Union[None, str, Iterable[str]] = None,
+                 ds_id: Union[None, str, Iterable[str]] = None) -> None:
+        super().__init__(inputs=inputs, outputs=inputs, mode=mode, ds_id=ds_id)
         self.in_list, self.out_list = True, True
         self.retain_graph = True
 
