@@ -13,4 +13,5 @@
 # limitations under the License.
 # ==============================================================================
 from fastestimator.trace import adapt, io, metric, xai
-from fastestimator.trace.trace import EvalEssential, Logger, TestEssential, Trace, TrainEssential, sort_traces
+from fastestimator.trace.trace import EvalEssential, Logger, TestEssential, Trace, TrainEssential, PerDSTrace, \
+    sort_traces
