@@ -14,17 +14,17 @@
 # ==============================================================================
 import time
 from collections import deque
-from typing import Iterable, List, Optional, Set, Union
+from typing import Iterable, List, Set, Union
 
-from natsort import humansorted
 import numpy as np
+from natsort import humansorted
 
 from fastestimator.backend.get_lr import get_lr
 from fastestimator.summary.summary import ValWithError
 from fastestimator.summary.system import System
 from fastestimator.util.data import Data
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import parse_modes, to_list, to_number, to_set
+from fastestimator.util.util import check_ds_id, check_io_names, parse_modes, to_list, to_number, to_set
 
 
 @traceable()
@@ -65,11 +65,14 @@ class Trace:
         mode: What mode(s) to execute this Trace in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
+        ds_id: What dataset id(s) to execute this Trace in. To execute regardless of ds_id, pass None. To execute in all
+            ds_ids except for a particular one, you can pass an argument like "!ds1".
     """
     system: System
     inputs: List[str]
     outputs: List[str]
     mode: Set[str]
+    ds_id: Set[str]
     # You can put keys in here to have them automatically added to EvalEssential without the user having to manually add
     # them to the Estimator monitor_names. See BestModelSaver for an example.
     fe_monitor_names: Set[str]
@@ -77,10 +80,12 @@ class Trace:
     def __init__(self,
                  inputs: Union[None, str, Iterable[str]] = None,
                  outputs: Union[None, str, Iterable[str]] = None,
-                 mode: Union[None, str, Iterable[str]] = None) -> None:
-        self.inputs = to_list(inputs)
-        self.outputs = to_list(outputs)
+                 mode: Union[None, str, Iterable[str]] = None,
+                 ds_id: Union[None, str, Iterable[str]] = None) -> None:
+        self.inputs = check_io_names(to_list(inputs))
+        self.outputs = check_io_names(to_list(outputs))
         self.mode = parse_modes(to_set(mode))
+        self.ds_id = check_ds_id(to_set(ds_id))
         self.fe_monitor_names = set()  # The use-case here is rare enough that we don't want to add this to the init sig
 
     def on_begin(self, data: Data) -> None:
@@ -131,6 +136,20 @@ class Trace:
         """
         pass
 
+    def get_outputs(self, ds_ids: Union[None, str, List[str]]) -> List[str]:
+        """What outputs will be generated by this trace.
+
+        You can ignore this unless you are designing a new trace that has special interactions between its outputs and
+        particular dataset ids.
+
+        Args:
+            ds_ids: The ds_ids under which this trace will execute.
+
+        Returns:
+            The outputs of this trace.
+        """
+        return list(self.outputs)
+
 
 @traceable()
 class TrainEssential(Trace):
@@ -301,7 +320,8 @@ class Logger(Trace):
         print(log_message)
 
 
-def sort_traces(traces: List[Trace], available_outputs: Optional[Set[str]] = None) -> List[Trace]:
+def sort_traces(traces: List[Trace], ds_ids: List[str], available_outputs: Union[None, str,
+                                                                                 Set[str]] = None) -> List[Trace]:
     """Sort traces to attempt to resolve any dependency issues.
 
     This is essentially a topological sort, but it doesn't seem worthwhile to convert the data into a graph
@@ -309,6 +329,7 @@ def sort_traces(traces: List[Trace], available_outputs: Optional[Set[str]] = Non
 
     Args:
         traces: A list of traces (not inside schedulers) to be sorted.
+        ds_ids: The ds_ids currently available to the traces.
         available_outputs: What output keys are already available for the traces to use. If None are provided, the
             sorting algorithm will assume that any keys not generated by traces are being provided by the system.
             This results in a less rigorous sorting.
@@ -320,7 +341,7 @@ def sort_traces(traces: List[Trace], available_outputs: Optional[Set[str]] = Non
         AssertionError: If Traces have circular dependencies or require input keys which are not available.
     """
     sorted_traces = []
-    trace_outputs = {output for trace in traces for output in trace.outputs}
+    trace_outputs = {output for trace in traces for output in trace.get_outputs(ds_ids=ds_ids)}
     if available_outputs is None:
         # Assume that anything not generated by a Trace is provided by the system
         available_outputs = {inp for trace in traces for inp in trace.inputs} - trace_outputs
@@ -335,7 +356,7 @@ def sort_traces(traces: List[Trace], available_outputs: Optional[Set[str]] = Non
     while trace_deque:
         trace = trace_deque.popleft()
         ins = set(trace.inputs)
-        outs = set(trace.outputs)
+        outs = set(trace.get_outputs(ds_ids=ds_ids))
         if not ins or isinstance(trace, (TrainEssential, EvalEssential, TestEssential)):
             sorted_traces.append(trace)
             available_outputs |= outs
@@ -355,7 +376,7 @@ def sort_traces(traces: List[Trace], available_outputs: Optional[Set[str]] = Non
     while intermediate_traces:
         trace = intermediate_traces.popleft()
         ins = set(trace.inputs)
-        outs = set(trace.outputs)
+        outs = set(trace.get_outputs(ds_ids=ds_ids))
         already_seen.add(trace)
         if ins <= available_outputs or (weak_sort and (ins - outs - available_outputs).isdisjoint(trace_outputs)):
             sorted_traces.append(trace)
@@ -372,3 +393,22 @@ def sort_traces(traces: List[Trace], available_outputs: Optional[Set[str]] = Non
                 [type(tr).__name__ for tr in already_seen])))
     sorted_traces.extend(list(end_traces))
     return sorted_traces
+
+
+class PerDSTrace(Trace):
+    def on_ds_begin(self, data: Data) -> None:
+        """Runs at the beginning of each dataset.
+
+        Args:
+            data: A dictionary through which traces can communicate with each other. Output here will not be logged.
+        """
+        pass
+
+    def on_ds_end(self, data: Data) -> None:
+        """Runs at the beginning of each dataset.
+
+        Args:
+            data: A dictionary through which traces can communicate with each other. Output here will be accumulated
+                across all available datasets and then logged during on_epoch_end.
+        """
+        pass
