@@ -20,7 +20,10 @@ import torch
 from torch.utils.data import DataLoader, Dataset
 
 import fastestimator as fe
+from fastestimator.dataset.batch_dataset import BatchDataset
+from fastestimator.dataset.numpy_dataset import NumpyDataset
 from fastestimator.op.numpyop import NumpyOp
+from fastestimator.op.numpyop.univariate import Minmax
 from fastestimator.op.tensorop import TensorOp
 from fastestimator.schedule import EpochScheduler
 from fastestimator.test.unittest_util import is_equal
@@ -38,10 +41,22 @@ class SampleTensorOp(TensorOp):
 
 class NumpyOpAdd1(NumpyOp):
     def forward(self, data, state):
-
         return data + 1
 
 
+class ListData(Dataset):
+    def __init__(self, ds, key1="x", key2="y"):
+        self.ds = ds
+        self.key1 = key1
+        self.key2 = key2
+
+    def __getitem__(self, idx):
+        return [{self.key1: self.ds[idx]["x"], self.key2: self.ds[idx]["y"]} for _ in range(5)]
+
+    def __len__(self):
+        return len(self.ds)
+
+
 class TorchCustomDataset(Dataset):
     def __init__(self, data):
         super().__init__()
@@ -86,15 +101,14 @@ class TestPipelineInit(unittest.TestCase):
 
     This test has dependency:
     * fe.schedule.schedule.get_current_items
-    * fe.schedule.schedule.EpochSceduler
+    * fe.schedule.schedule.EpochScheduler
     """
-    @classmethod
-    def setUpClass(cls):
-        cls.sample_tf_dataset = get_sample_tf_dataset()
-        cls.sample_torch_dataset = get_sample_torch_dataset()
-        cls.sample_torch_dataloader = get_sample_torch_dataloader()
-        cls.sample_numpy_op = SampleNumpyOp(inputs="x", outputs="x")
-        cls.sample_tensor_op = SampleTensorOp(inputs="x", outputs="x")
+    def setUp(self):
+        self.sample_tf_dataset = get_sample_tf_dataset()
+        self.sample_torch_dataset = get_sample_torch_dataset()
+        self.sample_torch_dataloader = get_sample_torch_dataloader()
+        self.sample_numpy_op = SampleNumpyOp(inputs="x", outputs="x")
+        self.sample_tensor_op = SampleTensorOp(inputs="x", outputs="x")
 
     def test_pipeline_init_tf_dataset_torch_dataloader_have_op_batch_size_num_process(self):
         dataset = {"tf_dataset": self.sample_tf_dataset, "dataloader": self.sample_torch_dataloader}
@@ -145,19 +159,19 @@ class TestPipelineInit(unittest.TestCase):
             try:
                 pipeline = fe.Pipeline(train_data=data, eval_data=data, test_data=data, ops=[self.sample_numpy_op])
             except:
-                self.fail("exception occur")
+                self.fail("exception occurred")
 
         with self.subTest("with batch_size not None"):
             try:
                 pipeline = fe.Pipeline(train_data=data, eval_data=data, test_data=data, batch_size=10)
             except:
-                self.fail("exception occur")
+                self.fail("exception occurred")
 
         with self.subTest("with num_process not None"):
             try:
                 pipeline = fe.Pipeline(train_data=data, eval_data=data, test_data=data, num_process=1)
             except:
-                self.fail("exception occur")
+                self.fail("exception occurred")
 
     def test_pipeline_init_torch_dataset_scheduler_have_op_batch_size_num_process(self):
         data = EpochScheduler(epoch_dict={1: self.sample_torch_dataset, 2: None})
@@ -166,19 +180,19 @@ class TestPipelineInit(unittest.TestCase):
             try:
                 pipeline = fe.Pipeline(train_data=data, eval_data=data, test_data=data, ops=[self.sample_numpy_op])
             except:
-                self.fail("exception occur")
+                self.fail("exception occurred")
 
         with self.subTest("with batch_size not None"):
             try:
                 pipeline = fe.Pipeline(train_data=data, eval_data=data, test_data=data, batch_size=10)
             except:
-                self.fail("exception occur")
+                self.fail("exception occurred")
 
         with self.subTest("with num_process not None"):
             try:
                 pipeline = fe.Pipeline(train_data=data, eval_data=data, test_data=data, num_process=1)
             except:
-                self.fail("exception occur")
+                self.fail("exception occurred")
 
     def test_pipeline_init_all_dataset_no_op_batch_size_num_process(self):
         dataset = {
@@ -192,7 +206,7 @@ class TestPipelineInit(unittest.TestCase):
                 try:
                     pipeline = fe.Pipeline(train_data=data, eval_data=data, test_data=data)
                 except:
-                    self.fail("exception occur")
+                    self.fail("exception occurred")
 
     def test_pipeline_init_torch_dataset_with_tensorop(self):
         data = self.sample_torch_dataset
@@ -213,11 +227,10 @@ class TestPipelineInit(unittest.TestCase):
 class TestPipelineGetModes(unittest.TestCase):
     """This test include:
     * fe.pipeline.Pipeline.get_modes
-    * fe.schedule.schedule.EpochSceduler
+    * fe.schedule.schedule.EpochScheduler
     """
-    @classmethod
-    def setUpClass(cls):
-        cls.sample_torch_dataset = get_sample_torch_dataset()
+    def setUp(self):
+        self.sample_torch_dataset = get_sample_torch_dataset()
 
     def test_pipeline_get_modes_no_scheduler(self):
         with self.subTest("train_data"):
@@ -256,11 +269,10 @@ class TestPipelineGetModes(unittest.TestCase):
 class TestPipelineGetEpochsWithData(unittest.TestCase):
     """This test include:
     * fe.pipeline.Pipeline.get_epochs_with_data
-    * fe.schedule.schedule.EpochSceduler
+    * fe.schedule.schedule.EpochScheduler
     """
-    @classmethod
-    def setUpClass(cls):
-        cls.sample_torch_dataset = get_sample_torch_dataset()
+    def setUp(self):
+        self.sample_torch_dataset = get_sample_torch_dataset()
 
     def test_pipeline_get_epochs_with_data_no_scheduler(self):
         pipeline = fe.Pipeline(train_data=self.sample_torch_dataset)
@@ -280,17 +292,16 @@ class TestPipelineGetEpochsWithData(unittest.TestCase):
         self.assertEqual(epochs, {1, 2})
 
 
-class TestPipelineBechmark(unittest.TestCase):
+class TestPipelineBenchmark(unittest.TestCase):
     """ This test has dependency on:
     * fe.pipeline.Pipeline.get_loader
     """
-    @classmethod
-    def setUpClass(cls):
-        cls.sample_tf_dataset = get_sample_tf_dataset()
-        cls.sample_torch_dataset = get_sample_torch_dataset()
-        cls.sample_torch_dataloader = get_sample_torch_dataloader()
+    def setUp(self):
+        self.sample_tf_dataset = get_sample_tf_dataset()
+        self.sample_torch_dataset = get_sample_torch_dataset()
+        self.sample_torch_dataloader = get_sample_torch_dataloader()
 
-    def test_pipeline_bechmark_smoke(self):
+    def test_pipeline_benchmark_smoke(self):
         dataset = {
             "tf_dataset": self.sample_tf_dataset,
             "torch_dataset": self.sample_torch_dataset,
@@ -300,11 +311,25 @@ class TestPipelineBechmark(unittest.TestCase):
         for data_type, data in dataset.items():
             with self.subTest("{}".format(data_type)):
                 pipeline = fe.Pipeline(train_data=data)
-
                 try:
                     pipeline.benchmark()
                 except:
-                    self.fail("exception occur")
+                    self.fail("exception occurred")
+
+    def test_multi_ds(self):
+        ds = {"ds_1": self.sample_torch_dataset, "ds_2": self.sample_torch_dataset}
+        pipeline = fe.Pipeline(train_data=ds)
+        pipeline.benchmark()
+
+    def test_multi_eval(self):
+        ds = {"ds_1": self.sample_torch_dataset, "ds_2": self.sample_torch_dataset}
+        pipeline = fe.Pipeline(eval_data=ds)
+        pipeline.benchmark(mode="eval")
+
+    def test_multi_test(self):
+        ds = {"ds_1": self.sample_torch_dataset, "ds_2": self.sample_torch_dataset}
+        pipeline = fe.Pipeline(test_data=ds)
+        pipeline.benchmark(mode="test")
 
 
 class TestPipelineTransform(unittest.TestCase):
@@ -312,10 +337,9 @@ class TestPipelineTransform(unittest.TestCase):
     * fe.schedule.schedule.get_current_items
     * fe.op.numpy.numpy.forward_numpy
     """
-    @classmethod
-    def setUpClass(cls):
-        cls.sample_data = {"x": np.array([1, 2, 3], dtype=np.float32)}
-        cls.sample_dataset = get_sample_torch_dataset()
+    def setUp(self):
+        self.sample_data = {"x": np.array([1, 2, 3], dtype=np.float32)}
+        self.sample_dataset = get_sample_torch_dataset()
 
     def test_pipeline_transform_no_ops(self):
         pipeline = fe.Pipeline()
@@ -329,13 +353,56 @@ class TestPipelineTransform(unittest.TestCase):
         ans = {"x": np.array([[1, 2, 3]], dtype=np.float32), "y": np.array([[2, 3, 4]], dtype=np.float32)}
         self.assertTrue(is_equal(data, ans))
 
+    def test_multi_train(self):
+        pipeline = fe.Pipeline(train_data=self.sample_dataset,
+                               ops=Minmax(inputs="x", outputs="x", ds_id=("ds_1", "ds_2")))
+        sample_data = {"x": np.array([0, 255], dtype=np.float32)}
+        data1 = pipeline.transform(data=sample_data, mode="train", ds_id="ds_1")
+        assert data1["x"].max() == 1.0
+        data2 = pipeline.transform(data=sample_data, mode="train", ds_id="ds_2")
+        assert data2["x"].max() == 1.0
+        data3 = pipeline.transform(data=sample_data, mode="train", ds_id="ds_3")
+        assert data3["x"].max() == 255
+
+    def test_multi_eval(self):
+        pipeline = fe.Pipeline(train_data=self.sample_dataset,
+                               ops=Minmax(inputs="x", outputs="x", ds_id=("!ds_1", "!ds_2")))
+        sample_data = {"x": np.array([0, 255], dtype=np.float32)}
+        data1 = pipeline.transform(data=sample_data, mode="eval", ds_id="ds_1")
+        assert data1["x"].max() == 255
+        data2 = pipeline.transform(data=sample_data, mode="eval", ds_id="ds_2")
+        assert data2["x"].max() == 255
+        data3 = pipeline.transform(data=sample_data, mode="eval", ds_id="ds_3")
+        assert data3["x"].max() == 1.0
+
+    def test_multi_test(self):
+        pipeline = fe.Pipeline(train_data=self.sample_dataset, ops=Minmax(inputs="x", outputs="x", ds_id="!ds_1"))
+        sample_data = {"x": np.array([0, 255], dtype=np.float32)}
+        data1 = pipeline.transform(data=sample_data, mode="test", ds_id="ds_1")
+        assert data1["x"].max() == 255
+        data2 = pipeline.transform(data=sample_data, mode="test", ds_id="ds_2")
+        assert data2["x"].max() == 1.0
+        data3 = pipeline.transform(data=sample_data, mode="test", ds_id="ds_3")
+        assert data3["x"].max() == 1.0
+
+    def test_multi_infer(self):
+        pipeline = fe.Pipeline(train_data=self.sample_dataset, ops=[Minmax(inputs="x", outputs="x", ds_id="!ds_1")])
+        sample_data = {"x": np.array([0, 255], dtype=np.float32)}
+        data1 = pipeline.transform(data=sample_data, mode="infer", ds_id="ds_1")
+        assert data1["x"].max() == 255
+        data2 = pipeline.transform(data=sample_data, mode="infer", ds_id="ds_2")
+        assert data2["x"].max() == 1.0
+        data3 = pipeline.transform(data=sample_data, mode="infer", ds_id="ds_3")
+        assert data3["x"].max() == 1.0
+        data4 = pipeline.transform(data=sample_data, mode="infer")
+        assert data4["x"].max() == 1.0
+
 
 class TestPipelineGetResults(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        cls.sample_tf_dataset = get_sample_tf_dataset()
-        cls.sample_torch_dataset = get_sample_torch_dataset()
-        cls.sample_torch_dataloader = get_sample_torch_dataloader()
+    def setUp(self):
+        self.sample_tf_dataset = get_sample_tf_dataset()
+        self.sample_torch_dataset = get_sample_torch_dataset()
+        self.sample_torch_dataloader = get_sample_torch_dataloader()
 
     def test_pipeline_get_result_tf_dataset_no_op(self):
         pipeline = fe.Pipeline(train_data=self.sample_tf_dataset)
@@ -393,7 +460,9 @@ class TestPipelineGetResults(unittest.TestCase):
         pipeline = fe.Pipeline(train_data=self.sample_torch_dataset,
                                eval_data=self.sample_torch_dataset,
                                ops=NumpyOpAdd1(inputs="x", outputs="y"),
-                               batch_size={"train": 2, "eval": 1})
+                               batch_size={
+                                   "train": 2, "eval": 1
+                               })
         data_train = pipeline.get_results(mode="train", epoch=1)
         data_eval = pipeline.get_results(mode="eval", epoch=1)
         data_train["x"] = data_train["x"].numpy()
@@ -405,6 +474,85 @@ class TestPipelineGetResults(unittest.TestCase):
         self.assertTrue(is_equal(data_train, ans_train))
         self.assertTrue(is_equal(data_eval, ans_eval))
 
+    def test_pipeline_get_results_list_data(self):
+        ds = self.sample_torch_dataset
+        ds = ListData(ds)
+        pipeline = fe.Pipeline(train_data=ds)
+        data = pipeline.get_results()
+        self.assertEqual(data["x"].size(0), 5)
+
+    def test_pipeline_get_results_batch_list_data(self):
+        ds1 = self.sample_torch_dataset
+        ds2 = ListData(ds1)
+        batch_ds = BatchDataset(datasets=(ds1, ds2), num_samples=(2, 1))
+        pipeline = fe.Pipeline(train_data=batch_ds)
+        data = pipeline.get_results()
+        self.assertEqual(data["x"].size(0), 7)
+
+    def test_pipeline_get_results_batch_list_data_disjoint_keys(self):
+        ds1 = self.sample_torch_dataset
+        ds2 = ListData(ds1, key1="x1", key2="y2")
+        batch_ds = BatchDataset(datasets=(ds1, ds2), num_samples=(5, 1))
+        pipeline = fe.Pipeline(train_data=batch_ds)
+        data = pipeline.get_results()
+        self.assertEqual(data["x"].size(0), 5)
+
+    def test_multi_train(self):
+        train_data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        train_ds = {"ds_1": train_data, "ds_2": train_data, "ds_3": train_data}
+        pipeline = fe.Pipeline(train_data=train_ds,
+                               batch_size=1,
+                               ops=Minmax(inputs="x", outputs="x", ds_id=("ds_1", "ds_2")))
+        data1 = pipeline.get_results(mode="train", ds_id="ds_3")
+        assert data1["x"].numpy().max() == 255
+        data2 = pipeline.get_results(mode="train", ds_id="ds_1")
+        assert data2["x"].numpy().max() == 1.0
+        data3 = pipeline.get_results(mode="train", ds_id="ds_2")
+        assert data3["x"].numpy().max() == 1.0
+
+    def test_multi_eval(self):
+        eval_data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        eval_ds = {"ds_1": eval_data, "ds_2": eval_data, "ds_3": eval_data}
+        pipeline = fe.Pipeline(eval_data=eval_ds,
+                               batch_size=1,
+                               ops=Minmax(inputs="x", outputs="x", ds_id=("!ds_1", "!ds_2")))
+        data1 = pipeline.get_results(mode="eval", ds_id="ds_3")
+        assert data1["x"].numpy().max() == 1.0
+        data2 = pipeline.get_results(mode="eval", ds_id="ds_1")
+        assert data2["x"].numpy().max() == 255
+        data3 = pipeline.get_results(mode="eval", ds_id="ds_2")
+        assert data3["x"].numpy().max() == 255
+
+    def test_multi_test(self):
+        test_data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        test_ds = {"ds_1": test_data, "ds_2": test_data, "ds_3": test_data}
+        pipeline = fe.Pipeline(test_data=test_ds, batch_size=1, ops=Minmax(inputs="x", outputs="x", ds_id="!ds_1"))
+        data1 = pipeline.get_results(mode="test", ds_id="ds_3")
+        assert data1["x"].numpy().max() == 1.0
+        data2 = pipeline.get_results(mode="test", ds_id="ds_1")
+        assert data2["x"].numpy().max() == 255
+        data3 = pipeline.get_results(mode="test", ds_id="ds_2")
+        assert data3["x"].numpy().max() == 1.0
+
+    def test_multi_train_scheduler(self):
+        train_ds = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        train_ds2 = NumpyDataset({"x": np.array([[0, 256], [256, 0]])})
+        train_data1 = train_ds
+        train_data2 = EpochScheduler({1: train_ds, 2: None})
+        train_data3 = EpochScheduler({1: train_ds, 2: train_ds2})
+        train_dataset_overall = {"ds_1": train_data1, "ds_2": train_data2, "ds_3": train_data3}
+        pipeline = fe.Pipeline(train_data=train_dataset_overall,
+                               batch_size=1,
+                               ops=Minmax(inputs="x", outputs="x", ds_id=("ds_1", "ds_2")))
+        data1 = pipeline.get_results(mode="train", ds_id="ds_3", epoch=1)
+        assert data1["x"].numpy().max() == 255
+        data2 = pipeline.get_results(mode="train", ds_id="ds_3", epoch=2)
+        assert data2["x"].numpy().max() == 256
+        data3 = pipeline.get_results(mode="train", ds_id="ds_1", epoch=1)
+        assert data3["x"].numpy().max() == 1.0
+        data4 = pipeline.get_results(mode="train", ds_id="ds_2", epoch=2)
+        assert data4 == []
+
 
 class TestPipelineGetLoader(unittest.TestCase):
     """ This test cover:
@@ -417,104 +565,94 @@ class TestPipelineGetLoader(unittest.TestCase):
     * fe.dataset.op_dataset.OpDataset
     * fe.pipeline.Pipeline._pad_batch_collate
     """
-    @classmethod
-    def setUpClass(cls):
-        cls.sample_tf_dataset = get_sample_tf_dataset()
-        cls.sample_torch_dataset = get_sample_torch_dataset()
-        cls.sample_torch_dataloader = get_sample_torch_dataloader()
+    def setUp(self):
+        self.sample_tf_dataset = get_sample_tf_dataset()
+        self.sample_torch_dataset = get_sample_torch_dataset()
+        self.sample_torch_dataloader = get_sample_torch_dataloader()
 
     def test_pipeline_get_loader_tf_dataset(self):
         pipeline = fe.Pipeline(train_data=self.sample_tf_dataset)
-        loader = pipeline.get_loader(mode="train")
-
-        self.assertEqual(loader, self.sample_tf_dataset)
+        with pipeline(mode="train") as loader:
+            self.assertEqual(loader, self.sample_tf_dataset)
 
     def test_pipeline_get_loader_torch_dataloader(self):
         pipeline = fe.Pipeline(train_data=self.sample_torch_dataloader)
-        loader = pipeline.get_loader(mode="train")
-
-        self.assertEqual(loader, self.sample_torch_dataloader)
+        with pipeline(mode="train") as loader:
+            self.assertEqual(loader, self.sample_torch_dataloader)
 
     def test_pipeline_get_loader_torch_dataset(self):
         pipeline = fe.Pipeline(train_data=self.sample_torch_dataset)
-        loader = pipeline.get_loader(mode="train")
-
-        with self.subTest("check loader type"):
-            self.assertIsInstance(loader, torch.utils.data.DataLoader)
-
-        with self.subTest("check data"):
-            results = []
-            for idx, batch in enumerate(loader, start=1):
-                results.append(batch)
-                if idx == 2:
-                    break
-            ans = [{
-                "x": torch.tensor([0], dtype=torch.float32), "y": torch.tensor([-99], dtype=torch.float32)
-            }, {
-                "x": torch.tensor([1], dtype=torch.float32), "y": torch.tensor([-98], dtype=torch.float32)
-            }]
-            self.assertTrue(is_equal(results, ans))
+        with pipeline(mode="train", shuffle=False) as loader:
+            with self.subTest("check loader type"):
+                self.assertIsInstance(loader, torch.utils.data.DataLoader)
+            with self.subTest("check data"):
+                results = []
+                for idx, batch in enumerate(loader, start=1):
+                    results.append(batch)
+                    if idx == 2:
+                        break
+                ans = [{
+                    "x": torch.tensor([0], dtype=torch.float32), "y": torch.tensor([-99], dtype=torch.float32)
+                }, {
+                    "x": torch.tensor([1], dtype=torch.float32), "y": torch.tensor([-98], dtype=torch.float32)
+                }]
+                self.assertTrue(is_equal(results, ans))
 
     def test_pipeline_get_loader_torch_dataset_with_batch_size(self):
         with self.subTest(shuffle=False):
             pipeline = fe.Pipeline(train_data=self.sample_torch_dataset, batch_size=2)
-            loader = pipeline.get_loader(mode="train", shuffle=False)
-
-            results = []
-            for idx, batch in enumerate(loader, start=1):
-                results.append(batch)
-                if idx == 2:
-                    break
-            ans = [{
-                "x": torch.tensor([[0], [1]], dtype=torch.float32),
-                "y": torch.tensor([[-99], [-98]], dtype=torch.float32)
-            },
-                   {
-                       "x": torch.tensor([[2], [3]], dtype=torch.float32),
-                       "y": torch.tensor([[-97], [-96]], dtype=torch.float32)
-                   }]
-            self.assertTrue(is_equal(results, ans))
+            with pipeline(mode="train", shuffle=False) as loader:
+                results = []
+                for idx, batch in enumerate(loader, start=1):
+                    results.append(batch)
+                    if idx == 2:
+                        break
+                ans = [{
+                    "x": torch.tensor([[0], [1]], dtype=torch.float32),
+                    "y": torch.tensor([[-99], [-98]], dtype=torch.float32)
+                },
+                       {
+                           "x": torch.tensor([[2], [3]], dtype=torch.float32),
+                           "y": torch.tensor([[-97], [-96]], dtype=torch.float32)
+                       }]
+                self.assertTrue(is_equal(results, ans))
 
         with self.subTest(shuffle=True):
             pipeline = fe.Pipeline(train_data=self.sample_torch_dataset, batch_size=2)
-            loader = pipeline.get_loader(mode="train", shuffle=True)
-
-            results = []
-            for idx, batch in enumerate(loader, start=1):
-                results.append(batch)
-                if idx == 2:
-                    break
-            wrong_ans = [{
-                "x": torch.tensor([[0], [1]], dtype=torch.float32),
-                "y": torch.tensor([[-99], [-98]], dtype=torch.float32)
-            },
-                         {
-                             "x": torch.tensor([[2], [3]], dtype=torch.float32),
-                             "y": torch.tensor([[-97], [-96]], dtype=torch.float32)
-                         }]
-
-            self.assertFalse(is_equal(results, wrong_ans))
+            with pipeline(mode="train", shuffle=True) as loader:
+                results = []
+                for idx, batch in enumerate(loader, start=1):
+                    results.append(batch)
+                    if idx == 2:
+                        break
+                wrong_ans = [{
+                    "x": torch.tensor([[0], [1]], dtype=torch.float32),
+                    "y": torch.tensor([[-99], [-98]], dtype=torch.float32)
+                },
+                             {
+                                 "x": torch.tensor([[2], [3]], dtype=torch.float32),
+                                 "y": torch.tensor([[-97], [-96]], dtype=torch.float32)
+                             }]
+                self.assertFalse(is_equal(results, wrong_ans))
 
         with self.subTest(shuffle=None):
             pipeline = fe.Pipeline(train_data=self.sample_torch_dataset, batch_size=2)
-            loader = pipeline.get_loader(mode="train", shuffle=None)
-
-            results = []
-            for idx, batch in enumerate(loader, start=1):
-                results.append(batch)
-                if idx == 2:
-                    break
-            wrong_ans = [{
-                "x": torch.tensor([[0], [1]], dtype=torch.float32),
-                "y": torch.tensor([[-99], [-98]], dtype=torch.float32)
-            },
-                         {
-                             "x": torch.tensor([[2], [3]], dtype=torch.float32),
-                             "y": torch.tensor([[-97], [-96]], dtype=torch.float32)
-                         }]
-
-            self.assertFalse(is_equal(results,
-                                      wrong_ans))  # if shuffle is None and has specify batch_size, it will shuffle
+            with pipeline(mode="train", shuffle=None) as loader:
+                results = []
+                for idx, batch in enumerate(loader, start=1):
+                    results.append(batch)
+                    if idx == 2:
+                        break
+                wrong_ans = [{
+                    "x": torch.tensor([[0], [1]], dtype=torch.float32),
+                    "y": torch.tensor([[-99], [-98]], dtype=torch.float32)
+                },
+                             {
+                                 "x": torch.tensor([[2], [3]], dtype=torch.float32),
+                                 "y": torch.tensor([[-97], [-96]], dtype=torch.float32)
+                             }]
+                self.assertFalse(is_equal(results,
+                                          wrong_ans))  # if shuffle is None and has specify batch_size, it will shuffle
 
     def test_pipeline_get_loader_torch_dataset_pad(self):
         """
@@ -526,11 +664,72 @@ class TestPipelineGetLoader(unittest.TestCase):
         """
         dataset = fe.dataset.NumpyDataset({"x": [np.ones((2, 1), dtype=np.float32), np.ones((1, 2), dtype=np.float32)]})
         pipeline = fe.Pipeline(train_data=dataset, pad_value=-1, batch_size=2)
-        loader = pipeline.get_loader(mode="train", shuffle=False)
-        for idx, batch in enumerate(loader, start=1):
-            result = batch
-            if idx == 1:
-                break
+        with pipeline(mode="train", shuffle=False) as loader:
+            for idx, batch in enumerate(loader, start=1):
+                result = batch
+                if idx == 1:
+                    break
+            ans = {"x": torch.tensor([[[1, -1], [1, -1]], [[1, 1], [-1, -1]]], dtype=torch.float32)}
+            self.assertTrue(is_equal(ans, result))
 
-        ans = {"x": torch.tensor([[[1, -1], [1, -1]], [[1, 1], [-1, -1]]], dtype=torch.float32)}
-        self.assertTrue(is_equal(ans, result))
+    def test_pipeline_nested_loaders(self):
+        dataset = fe.dataset.NumpyDataset({"x": [np.ones((2, 1), dtype=np.float32), np.ones((1, 2), dtype=np.float32)]})
+        pipeline = fe.Pipeline(train_data=dataset, pad_value=-1, batch_size=2)
+        with self.subTest("With Call"):
+            with self.assertRaises(ValueError):
+                with pipeline(mode='train') as loader1:
+                    with pipeline(mode='eval') as loader2:
+                        print(loader1)
+                        print(loader2)
+        with self.subTest("Without Call"):
+            with self.assertRaises(ValueError):
+                with pipeline(mode='train') as loader1:
+                    with pipeline as loader2:
+                        print(loader1)
+                        print(loader2)
+
+    def test_pipeline_nested_call(self):
+        dataset = fe.dataset.NumpyDataset({"x": [np.ones((2, 1), dtype=np.float32), np.ones((1, 2), dtype=np.float32)]})
+        pipeline = fe.Pipeline(train_data=dataset, pad_value=-1, batch_size=2)
+        with self.assertRaises(ValueError):
+            with pipeline(mode='train') as loader1:
+                pipeline(mode='train')
+                print(loader1)
+
+
+class TestPipelineNames(unittest.TestCase):
+    def test_forbidden_names_none(self):
+        data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        train_ds = {None: data}
+        with self.assertRaises(AssertionError):
+            fe.Pipeline(train_data=train_ds)
+
+    def test_forbidden_names_esc(self):
+        data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        train_ds = {"!ds1": data}
+        with self.assertRaises(AssertionError):
+            fe.Pipeline(train_data=train_ds)
+
+    def test_forbidden_names_semi(self):
+        data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        train_ds = {"ds1;": data}
+        with self.assertRaises(AssertionError):
+            fe.Pipeline(train_data=train_ds)
+
+    def test_forbidden_names_colon(self):
+        data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        train_ds = {"ds1:": data}
+        with self.assertRaises(AssertionError):
+            fe.Pipeline(train_data=train_ds)
+
+    def test_forbidden_names_empty(self):
+        data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        train_ds = {"": data}
+        with self.assertRaises(AssertionError):
+            fe.Pipeline(train_data=train_ds)
+
+    def test_forbidden_names_pipe(self):
+        data = NumpyDataset({"x": np.array([[0, 255], [255, 0]])})
+        train_ds = {"ds|ds1": data}
+        with self.assertRaises(AssertionError):
+            fe.Pipeline(train_data=train_ds)
